@@ -83,8 +83,9 @@
 #' @param boot Whether to use bootstrap to obtain the confidence
 #'   interval for hazard ratio. Defaults to \code{FALSE}.
 #' @param n_boot The number of bootstrap samples.
-#' @param seed The seed to reproduce the bootstrap results. The default is 
-#'   `NA`, in which case, the seed from the environment will be used.
+#' @param seed The seed to reproduce the bootstrap results.
+#' @param nthreads The number of threads to use in bootstrapping (0 means 
+#'   the default RcppParallel behavior)
 #'
 #' @details The hazard ratio and confidence interval under a no-switching 
 #' scenario are obtained as follows:
@@ -121,13 +122,10 @@
 #'
 #' @return A list with the following components:
 #'
-#' * \code{logrank_pvalue}:  The two-sided p-value of the log-rank test 
-#'   for the ITT analysis.
+#' * \code{pvalue}: The two-sided p-value.
 #'
-#' * \code{cox_pvalue}: The two-sided p-value for treatment effect based on
-#'   the weighted Cox model excluding data after treatment switch. 
-#'   If \code{boot} is \code{TRUE}, this value represents the 
-#'   bootstrap p-value.
+#' * \code{pvalue_type}: The type of two-sided p-value for treatment effect, 
+#'   i.e., "Cox model" or "bootstrap".
 #'
 #' * \code{hr}: The estimated hazard ratio from the Cox model.
 #'
@@ -196,7 +194,7 @@
 #' library(dplyr)
 #' 
 #' sim1 <- tssim(
-#'   tdxo = 1, coxo = 1, allocation1 = 1, allocation2 = 1,
+#'   tdxo = TRUE, coxo = TRUE, allocation1 = 1, allocation2 = 1,
 #'   p_X_1 = 0.3, p_X_0 = 0.3, 
 #'   rate_T = 0.002, beta1 = -0.5, beta2 = 0.3, 
 #'   gamma0 = 0.3, gamma1 = -0.9, gamma2 = 0.7, gamma3 = 1.1, gamma4 = -0.8,
@@ -246,92 +244,62 @@ ipcw <- function(data, id = "id", stratum = "", tstart = "tstart",
                  trunc = 0, trunc_upper_only = TRUE,
                  swtrt_control_only = TRUE, 
                  alpha = 0.05, ties = "efron", 
-                 boot = FALSE, n_boot = 1000, seed = NA) {
+                 boot = FALSE, n_boot = 1000, seed = 0,
+                 nthreads = 0) {
   
-  rownames(data) = NULL
-  
-  elements = c(id, stratum, tstart, tstop, event, treat, swtrt)
-  elements = unique(elements[elements != "" & elements != "none"])
-  fml = formula(paste("~", paste(elements, collapse = "+")))
-  mf = model.frame(fml, data = data, na.action = na.omit)
-  
-  rownum = as.integer(rownames(mf))
-  df = data[rownum,]
-  
-  nvar = length(base_cov)
-  if (missing(base_cov) || is.null(base_cov) || (nvar == 1 && (
-    base_cov[1] == "" || tolower(base_cov[1]) == "none"))) {
-    p = 0
-  } else {
-    fml1 = formula(paste("~", paste(base_cov, collapse = "+")))
-    vnames = rownames(attr(terms(fml1), "factors"))
-    p = length(vnames)
+  # validate input
+  if (!inherits(data, "data.frame")) {
+    stop("Input 'data' must be a data frame");
   }
   
-  if (p >= 1) {
-    mf1 <- model.frame(fml1, data = df, na.action = na.pass)
-    mm <- model.matrix(fml1, mf1)
-    colnames(mm) = make.names(colnames(mm))
-    varnames = colnames(mm)[-1]
-    for (i in 1:length(varnames)) {
-      if (!(varnames[i] %in% names(df))) {
-        df[,varnames[i]] = mm[,varnames[i]]
-      }
+  if (inherits(data, "data.table") || inherits(data, "tbl") || 
+      inherits(data, "tbl_df")) {
+    df <- as.data.frame(data)
+  } else {
+    df <- data
+  }
+  
+  for (nm in c(id, tstart, tstop, event, treat, swtrt, swtrt_time)) {
+    if (!is.character(nm) || length(nm) != 1) {
+      stop(paste(nm, "must be a single character string."));
     }
-  } else {
-    varnames = ""
   }
   
-  nvar2 = length(numerator)
-  if (missing(numerator) || is.null(numerator) || (nvar2 == 1 && (
-    numerator[1] == "" || tolower(numerator[1]) == "none"))) {
-    p2 = 0
-  } else {
-    fml2 = formula(paste("~", paste(numerator, collapse = "+")))
-    vnames2 = rownames(attr(terms(fml2), "factors"))
-    p2 = length(vnames2)
+  # Respect user-requested number of threads (best effort)
+  if (nthreads > 0) {
+    n_physical_cores <- parallel::detectCores(logical = FALSE)
+    RcppParallel::setThreadOptions(min(nthreads, n_physical_cores))
   }
   
-  if (p2 >= 1) {
-    mf2 <- model.frame(fml2, data = df, na.action = na.pass)
-    mm2 <- model.matrix(fml2, mf2)
-    colnames(mm2) = make.names(colnames(mm2))
-    varnames2 = colnames(mm2)[-1]
-    for (i in 1:length(varnames2)) {
-      if (!(varnames2[i] %in% names(df))) {
-        df[,varnames2[i]] = mm2[,varnames2[i]]
-      }
-    }
-  } else {
-    varnames2 = ""
-  }
+  # select complete cases for the relevant variables
+  elements = unique(c(id, stratum, tstart, tstop, event, treat, swtrt))
+  elements = elements[elements != ""]
+  fml_all <- formula(paste("~", paste(elements, collapse = "+")))
+  var_all <- all.vars(fml_all)
+  rows_ok <- which(complete.cases(df[, var_all, drop = FALSE]))
+  if (length(rows_ok) == 0) 
+    stop("No complete cases found for the specified variables.")
+  df <- df[rows_ok, , drop = FALSE]
+
+  # process covariate specifications
+  res1 <- process_cov(base_cov, df)
+  df <- res1$df
+  vnames    <- res1$vnames
+  varnames  <- res1$varnames
   
-  nvar3 = length(denominator)
-  if (missing(denominator) || is.null(denominator) || (nvar3 == 1 && (
-    denominator[1] == "" || tolower(denominator[1]) == "none"))) {
-    p3 = 0
-  } else {
-    fml3 = formula(paste("~", paste(denominator, collapse = "+")))
-    vnames3 = rownames(attr(terms(fml3), "factors"))
-    p3 = length(vnames3)
-  }
+  res2 <- process_cov(numerator, df)
+  df <- res2$df
+  vnames2    <- res2$vnames
+  varnames2  <- res2$varnames
   
-  if (p3 >= 1) {
-    mf3 <- model.frame(fml3, data = df, na.action = na.pass)
-    mm3 <- model.matrix(fml3, mf3)
-    colnames(mm3) = make.names(colnames(mm3))
-    varnames3 = colnames(mm3)[-1]
-    for (i in 1:length(varnames3)) {
-      if (!(varnames3[i] %in% names(df))) {
-        df[,varnames3[i]] = mm3[,varnames3[i]]
-      }
-    }
-  } else {
-    varnames3 = ""
-  }
+  res3 <- process_cov(denominator, df)
+  df <- res3$df
+  vnames3    <- res3$vnames
+  varnames3  <- res3$varnames
   
+  # call the core cpp function
   out <- ipcwcpp(
-    data = df, id = id, stratum = stratum, tstart = tstart,
+    df = df, id = id, stratum = stratum, tstart = tstart,
     tstop = tstop, event = event, treat = treat, 
     swtrt = swtrt, swtrt_time = swtrt_time, 
     base_cov = varnames, numerator = varnames2, denominator = varnames3,
@@ -355,13 +323,13 @@ ipcw <- function(data, id = "id", stratum = "", tstart = "tstart",
   out$data_outcome$uid <- NULL
   out$data_outcome$ustratum <- NULL
   
-  if (p >= 1) {
+  if (length(vnames) > 0) {
     add_vars <- setdiff(vnames, varnames)
     if (length(add_vars) > 0) {
-      out$data_outcome <- 
-        merge(out$data_outcome, 
-              dfu[, c(id, add_vars)], 
-              by = id, all.x = TRUE, sort = FALSE)
+      out$data_outcome <- merge_append(
+        A = out$data_outcome, B = dfu, 
+        by_vars = id, new_vars = add_vars,
+        overwrite = FALSE, first_match = FALSE)
     }
     
     del_vars <- setdiff(varnames, vnames)
@@ -377,7 +345,7 @@ ipcw <- function(data, id = "id", stratum = "", tstart = "tstart",
     out$data_switch[[h]]$data$ustratum <- NULL
   }
   
-  if (p3 >= 1) {
+  if (length(vnames3) > 0) {
     # exclude observations after treatment switch
     data1 <- df[!df[[swtrt]] | df[[tstart]] < df[[swtrt_time]], ]
     
@@ -398,24 +366,23 @@ ipcw <- function(data, id = "id", stratum = "", tstart = "tstart",
     if (length(avars) > 0) {
       if (logistic_switching_model) {
         for (h in 1:K) {
-          out$data_switch[[h]]$data <- 
-            merge(out$data_switch[[h]]$data, 
-                  data1[, c(id, "tstart", "tstop", avars)], 
-                  by = c(id, "tstart", "tstop"), all.x = TRUE, sort = FALSE)
+          out$data_switch[[h]]$data <- merge_append(
+            A = out$data_switch[[h]]$data, B = data1, 
+            by_vars = c(id, "tstart", "tstop"), new_vars = avars,
+            overwrite = FALSE, first_match = FALSE)
         }
       } else {
         # replicate event times within each subject
         cut <- sort(unique(data1$tstop[data1[[event]] == 1]))
         a1 <- survsplit(data1$tstart, data1$tstop, cut)
-        data2 <- data1[a1$row+1,]
+        data2 <- data1[a1$row  + 1, ]
         data2$tstart = a1$start
         data2$tstop = a1$end
-        
         for (h in 1:K) {
-          out$data_switch[[h]]$data <- 
-            merge(out$data_switch[[h]]$data, 
-                  data2[, c(id, "tstart", "tstop", avars)], 
-                  by = c(id, "tstart", "tstop"), all.x = TRUE, sort = FALSE)
+          out$data_switch[[h]]$data <- merge_append(
+            A = out$data_switch[[h]]$data, B = data2, 
+            by_vars = c(id, "tstart", "tstop"), new_vars = avars,
+            overwrite = FALSE, first_match = FALSE)
         }
       }
     }
@@ -428,30 +395,23 @@ ipcw <- function(data, id = "id", stratum = "", tstart = "tstart",
     }
   }
   
-  
   # convert treatment back to a factor variable if needed
   if (is.factor(data[[treat]])) {
     levs = levels(data[[treat]])
+    mf <- function(x) factor(x, levels = c(1,2), labels = levs)
     
-    out$event_summary[[treat]] <- factor(out$event_summary[[treat]], 
-                                         levels = c(1,2), labels = levs)
-    
-    for (h in 1:2) {
-      out$data_switch[[h]][[treat]] <- factor(
-        out$data_switch[[h]][[treat]], levels = c(1,2), labels = levs)
+    # apply mf to a set of data.frames with a column named `treat`
+    for (nm in c("event_summary", "weight_summary", "data_outcome", 
+                 "km_outcome")) {
+      out[[nm]][[treat]] <- mf(out[[nm]][[treat]])
     }
     
-    out$data_outcome[[treat]] <- factor(out$data_outcome[[treat]], 
-                                        levels = c(1,2), labels = levs)
-    
-    out$weight_summary[[treat]] <- factor(out$weight_summary[[treat]], 
-                                          levels = c(1,2), labels = levs)
-    
-    out$km_outcome[[treat]] <- factor(out$km_outcome[[treat]], 
-                                      levels = c(1,2), labels = levs)
+    # and for the list-of-lists
+    out$data_switch <- lapply(out$data_switch, function(x) { 
+      x[[treat]] <- mf(x[[treat]]); x 
+    })
   }
   
-  
   out$settings <- list(
     data = data, id = id, stratum = stratum, tstart = tstart, 
     tstop = tstop, event = event, treat = treat, swtrt = swtrt, 
