@@ -1,5 +1,18 @@
+#include <Rcpp.h>
+
+#include <boost/random.hpp>
+
 #include "utilities.h"
-using namespace Rcpp;
+#include "dataframe_list.h"
+
+#include <algorithm>
+#include <cmath>
+#include <cstddef>
+#include <cstdint>
+#include <random>
+#include <stdexcept>
+#include <vector>
+
 
 //' @title Simulate Data for Treatment Switching
 //' @description Simulates data for studies involving treatment switching, 
@@ -244,167 +257,170 @@ using namespace Rcpp;
 //'
 //' @export
 // [[Rcpp::export]]
-List tssim(const bool tdxo = 0, 
-           const bool coxo = 1, 
-           const int allocation1 = 1,
-           const int allocation2 = 1,
-           const double p_X_1 = NA_REAL, 
-           const double p_X_0 = NA_REAL, 
-           const double rate_T = NA_REAL, 
-           const double beta1 = NA_REAL, 
-           const double beta2 = NA_REAL, 
-           const double gamma0 = NA_REAL, 
-           const double gamma1 = NA_REAL, 
-           const double gamma2 = NA_REAL, 
-           const double gamma3 = NA_REAL, 
-           const double gamma4 = NA_REAL,
-           const double zeta0 = NA_REAL, 
-           const double zeta1 = NA_REAL, 
-           const double zeta2 = NA_REAL, 
-           const double zeta3 = NA_REAL, 
-           const double alpha0 = NA_REAL, 
-           const double alpha1 = NA_REAL, 
-           const double alpha2 = NA_REAL, 
-           const double theta1_1 = NA_REAL, 
-           const double theta1_0 = NA_REAL, 
-           const double theta2 = NA_REAL,
-           const double rate_C = NA_REAL,
-           const NumericVector& accrualTime = 0,
-           const NumericVector& accrualIntensity = NA_REAL,
-           const double followupTime = NA_REAL,
-           const bool fixedFollowup = 0,
-           const double plannedTime = NA_REAL,
-           const double days = NA_INTEGER,
-           const int n = NA_INTEGER, 
-           const int NSim = 1000, 
-           const int seed = NA_INTEGER) {
+Rcpp::List tssim(const bool tdxo = false, 
+                 const bool coxo = true, 
+                 const int allocation1 = 1,
+                 const int allocation2 = 1,
+                 const double p_X_1 = 0.3, 
+                 const double p_X_0 = 0.3, 
+                 const double rate_T = 0.002, 
+                 const double beta1 = -0.5, 
+                 const double beta2 = 0.3, 
+                 const double gamma0 = 0.3, 
+                 const double gamma1 = -0.9, 
+                 const double gamma2 = 0.7, 
+                 const double gamma3 = 1.1, 
+                 const double gamma4 = -0.8,
+                 const double zeta0 = -3.5, 
+                 const double zeta1 = 0.5, 
+                 const double zeta2 = 0.2, 
+                 const double zeta3 = -0.4, 
+                 const double alpha0 = 0.5, 
+                 const double alpha1 = 0.5, 
+                 const double alpha2 = 0.4, 
+                 const double theta1_1 = -0.4, 
+                 const double theta1_0 = -0.4, 
+                 const double theta2 = 0.2,
+                 const double rate_C = 0.0000855,
+                 const Rcpp::NumericVector& accrualTime = 0,
+                 const Rcpp::NumericVector& accrualIntensity = NA_REAL,
+                 const double followupTime = NA_REAL,
+                 const bool fixedFollowup = false,
+                 const double plannedTime = 1350.0,
+                 const double days = 30.0,
+                 const int n = 500,
+                 const int NSim = 1000, 
+                 const int seed = 0) {
   
-  if (allocation1 < 1) {
-    stop("allocation1 must be a positive integer");
+  std::vector<double> accTime = Rcpp::as<std::vector<double>>(accrualTime);
+  std::vector<double> accRate = Rcpp::as<std::vector<double>>(accrualIntensity);
+  
+  if (allocation1 < 1) 
+    throw std::invalid_argument("allocation1 must be a positive integer");
+  if (allocation2 < 1) 
+    throw std::invalid_argument("allocation2 must be a positive integer");
+  
+  if (p_X_1 <= 0 || p_X_1 >= 1) 
+    throw std::invalid_argument("p_X_1 must lie between 0 and 1");
+  if (p_X_0 <= 0 || p_X_0 >= 1) 
+    throw std::invalid_argument("p_X_0 must lie between 0 and 1");
+  
+  if (rate_T <= 0) throw std::invalid_argument("rate_T must be positive");
+  if (rate_C < 0) throw std::invalid_argument("rate_C must be nonnegative");
+  
+  if (accTime[0] != 0) 
+    throw std::invalid_argument("accrualTime must start with 0");
+  if (accTime.size() > 1) {
+    for (std::size_t i = 1; i < accTime.size(); ++i) {
+      double prev = accTime[i - 1];
+      double cur  = accTime[i];
+      // If either element is NaN, skip the comparison
+      if (std::isnan(prev) || std::isnan(cur)) continue;
+      if (cur <= prev) {
+        throw std::invalid_argument("accrualTime should be increasing");
+      }
+    }
   }
   
-  if (allocation2 < 1) {
-    stop("allocation2 must be a positive integer");
+  // Check accrualIntensity has no missing values (NaN)
+  if (std::any_of(accRate.begin(), accRate.end(),
+                  [](double x){ return std::isnan(x); })) {
+    throw std::invalid_argument("accrualIntensity must be provided");
   }
   
-  if (p_X_1 <= 0 || p_X_1 >= 1) {
-    stop("p_X_1 must lie between 0 and 1");
+  // length check
+  if (accTime.size() != accRate.size()) {
+    throw std::invalid_argument(
+        "accrualTime must have the same length as accrualIntensity");
   }
   
-  if (p_X_0 <= 0 || p_X_0 >= 1) {
-    stop("p_X_0 must lie between 0 and 1");
-  }
-  
-  if (rate_T <= 0) {
-    stop("rate_T must be positive");
-  }
-  
-  if (rate_C < 0) {
-    stop("rate_C must be nonnegative");
-  }
-  
-  if (accrualTime[0] != 0) {
-    stop("accrualTime must start with 0");
-  }
-  
-  if (accrualTime.size() > 1 && is_true(any(diff(accrualTime) <= 0))) {
-    stop("accrualTime should be increasing");
-  }
-  
-  if (is_true(any(is_na(accrualIntensity)))) {
-    stop("accrualIntensity must be provided");
-  }
-  
-  if (accrualTime.size() != accrualIntensity.size()) {
-    stop("accrualTime must have the same length as accrualIntensity");
-  }
-  
-  if (is_true(any(accrualIntensity < 0))) {
-    stop("accrualIntensity must be non-negative");
+  // accrualIntensity: must be provided (no NaN) and non-negative
+  for (std::size_t i = 0; i < accRate.size(); ++i) {
+    double v = accRate[i];
+    if (std::isnan(v)) {
+      throw std::invalid_argument("accrualIntensity must be provided");
+    }
+    if (v < 0.0) {
+      throw std::invalid_argument("accrualIntensity must be non-negative");
+    }
   }
   
+  // fixed follow-up checks
   if (fixedFollowup) {
-    if (R_isnancpp(followupTime)) {
-      stop("followupTime must be provided for fixed follow-up");
+    if (std::isnan(followupTime)) {
+      throw std::invalid_argument("followupTime must be provided for fixed follow-up");
     }
-    
-    if (followupTime <= 0) {
-      stop("followupTime must be positive for fixed follow-up");
+    if (followupTime <= 0.0) {
+      throw std::invalid_argument("followupTime must be positive for fixed follow-up");
     }
   }
   
-  if (R_isnancpp(plannedTime)) {
-    stop("plannedTime must be provided");
-  }
+  // plannedTime checks
+  if (std::isnan(plannedTime)) 
+    throw std::invalid_argument("plannedTime must be provided");
+  if (plannedTime <= 0.0) throw std::invalid_argument("plannedTime must be positive");
+
+  // days, n, NSim checks
+  if (days <= 0.0) throw std::invalid_argument("days must be positive");
+  if (n <= 0) throw std::invalid_argument("n must be positive");
+  if (NSim <= 0) throw std::invalid_argument("NSim must be positive");
   
-  if (plannedTime <= 0) {
-    stop("plannedTime must be positive");
-  }
+  // random number generator
+  boost::random::mt19937_64 rng(seed);
   
-  if (days <= 0) {
-    stop("days must be positive");
-  }
+  // distributions reused
+  boost::random::uniform_real_distribution<double> unif(0.0, 1.0);
+  boost::random::exponential_distribution<double> expo(1.0);
   
-  if (n <= 0) {
-    stop("n must be positive");
-  }
+  int maxFollowup = static_cast<int>(std::ceil(plannedTime / days));
+  int K = n * maxFollowup;
+  std::vector<DataFrameCpp> sims(NSim);
   
-  if (NSim <= 0) {
-    stop("NSim must be positive");
-  }
-  
-  if (seed != NA_INTEGER) set_seed(seed);
-  int maxFollowup = static_cast<int>(std::ceil(plannedTime/days));
-  int K = n*maxFollowup;
-  List result(NSim);
-  
-  for (int iter=0; iter<NSim; iter++) {
-    IntegerVector idx(K), trtrandx(K), bprogx(K), tpointx(K);
-    IntegerVector Lx(K), Llagx(K), Zx(K), Zlagx(K);
-    IntegerVector Ax(K), Alagx(K), Alag2x(K), eventx(K);
-    IntegerVector diedx(K), progressedx(K), xox(K);
-    NumericVector tstartx(K), tstopx(K), timeOSx(K), timePDx(K);
-    NumericVector xotimex(K, NA_REAL), censor_timex(K);
-    NumericVector arrivalTimex(K);
+  for (int iter = 0; iter < NSim; ++iter) {
+    std::vector<int> idx(K), trtrandx(K), tpointx(K);
+    std::vector<int> bprogx(K), Lx(K), Llagx(K), Zx(K), Zlagx(K);
+    std::vector<int> Ax(K), Alagx(K), Alag2x(K), eventx(K);
+    std::vector<int> diedx(K), progressedx(K), xox(K);
+    std::vector<double> tstartx(K), tstopx(K), timeOSx(K), timePDx(K);
+    std::vector<double> xotimex(K, NaN), censor_timex(K);
+    std::vector<double> arrivalTimex(K);
     
-    int id, trtrand, bprog, tpoint, L, Llag, Z, Zlag, A, Alag, Alag2, event;
-    double arrivalTime, tstart, tstop;
     double b1 = allocation1, b2 = allocation2;
     double enrollt = 0;
     int k = 0;
-    for (int i=1; i<=n; i++) { // i = subject index
-      id = i;
+    for (int i = 1; i <= n; ++i) { // subject index (1..n)
+      int id = i;
       
       // generate accrual time
-      double u = R::runif(0,1);
-      enrollt = qtpwexpcpp1(u, accrualTime, accrualIntensity, enrollt, 1, 0);
-      arrivalTime = std::ceil(enrollt);
+      enrollt = qtpwexpcpp1(unif(rng), accTime, accRate, enrollt, 1, 0);
+      double arrivalTime = std::ceil(enrollt);
       
       // stratified block randomization
-      u = R::runif(0,1);
-      if (u <= b1/(b1+b2)) {
-        trtrand = 1;
-        b1--;
-      } else {
-        trtrand = 0;
-        b2--;
-      }
-      if (b1 == 0 && b2 == 0) {
-        b1 = allocation1; b2 = allocation2;
-      }
+      // stratified block randomization
+      int trtrand;
+      if (unif(rng) <= b1 / (b1 + b2)) { trtrand = 1; b1 -= 1.0; }
+      else { trtrand = 0; b2 -= 1.0; }
+      if (b1 == 0.0 && b2 == 0.0) { b1 = allocation1; b2 = allocation2; }
       
-      bprog = (R::runif(0,1) <= p_X_1*trtrand + p_X_0*(1-trtrand));
-      double T = R::rexp(1.0/(rate_T*exp(beta1*trtrand + beta2*bprog)));
-      T = std::round(T);
-      if (T == 0) T = 1;
+      // baseline prognostic bprog (Bernoulli)
+      double p_bprog = (trtrand == 1) ? p_X_1 : p_X_0;
+      int bprog = (unif(rng) <= p_bprog) ? 1 : 0;
       
+      // event time T from exponential with rate = rate_T * exp(...)
+      double rate_this = rate_T * std::exp(beta1 * trtrand + beta2 * bprog);
+      double T = std::round(expo(rng) / rate_this);
+      if (T == 0.0) T = 1.0;
+      
+      // follow-up and cycles as before...
       double fu = std::max(plannedTime - arrivalTime, 0.0);
       if (fixedFollowup) fu = std::min(followupTime, fu);
-      int followup = static_cast<int>(std::floor(fu/days));
+      int followup = static_cast<int>(std::floor(fu / days));
       
-      tpoint = followup; L = 0; Llag = 0; Z = 0; A = 0; Alag = 0;
-      for (int j=1; j<=followup; j++) { // j = cycle index
-        tpoint = j; tstart = days*(j-1);
+      int tpoint = followup; 
+      int L = 0, Llag = 0, Z = 0, Zlag = 0, A = 0, Alag = 0, Alag2 = 0;
+      for (int j = 1; j <= followup; ++j) { // j = cycle index
+        tpoint = j; 
+        double tstart = days * (j - 1);
         if (j == 1) {
           Llag = 0; Zlag = 0; Alag2 = 0; Alag = 0;
         } else {
@@ -412,30 +428,35 @@ List tssim(const bool tdxo = 0,
         }
         
         // generate time-dependent covariate
-        L = (R::runif(0,1) <= R::plogis(gamma0 + gamma1*Alag + gamma2*Llag + 
-          gamma3*bprog + gamma4*trtrand, 0, 1, 1, 0));
+        double probL = boost_plogis(gamma0 + gamma1 * Alag + gamma2 * Llag +
+                                    gamma3 * bprog + gamma4 * trtrand);
+        L = (unif(rng) <= probL) ? 1 : 0;
         
-        if (T <= days*j) { // died in cycle j, complete data for the subject
+        double tstop;
+        int event;
+        if (T <= days * j) { // died in cycle j, complete data for the subject
           tstop = T; event = 1;
-          Z = NA_INTEGER; A = NA_INTEGER;
+          Z = INT_MIN; A = INT_MIN;
         } else { // alive at the end of cycle j, continue to the next cycle
-          tstop = days*j; event = 0;
+          tstop = days * j; event = 0;
           
           // generate disease progression status
           if (Zlag == 0) {
-            Z = (R::runif(0,1) <= R::plogis(zeta0 + zeta1*L + zeta2*bprog + 
-              zeta3*trtrand, 0, 1, 1, 0));
+            double probZ = boost_plogis(zeta0 + zeta1 * L + zeta2 * bprog + 
+                                        zeta3 * trtrand);
+            Z = (unif(rng) <= probZ) ? 1 : 0;
           } else {
             Z = 1;
           }
           
           // generate treatment switching status
           if (Alag == 0) {
-            if (((tdxo == 0 && Z == 1 && Zlag == 0) || 
-                (tdxo == 1 && Z == 1)) && 
-                ((coxo == 1 && trtrand == 0) || (coxo == 0))) {
-              A = (R::runif(0,1) <= R::plogis(alpha0 + alpha1*L + 
-                alpha2*bprog, 0, 1, 1, 0));
+            bool condition_for_switch = ((tdxo == 0 && Z == 1 && Zlag == 0) ||
+                                         (tdxo == 1 && Z == 1)) &&
+                                         ((coxo == 1 && trtrand == 0) || (coxo == 0));
+            if (condition_for_switch) {
+              double probA = boost_plogis(alpha0 + alpha1 * L + alpha2 * bprog);
+              A = (unif(rng) <= probA) ? 1 : 0;
             } else {
               A = 0;
             }
@@ -444,8 +465,9 @@ List tssim(const bool tdxo = 0,
           }
           
           // update survival time
-          double theta1 = theta1_1*trtrand + theta1_0*(1-trtrand);
-          T = days*j + (T - days*j)*exp(-theta1*(A-Alag) - theta2*(L-Llag));
+          double theta1 = theta1_1 * trtrand + theta1_0 * (1 - trtrand);
+          T = days * j + (T - days * j) * std::exp(-theta1 * (A - Alag) 
+                                                     - theta2 * (L - Llag));
           T = std::round(T);
         }
         
@@ -466,25 +488,24 @@ List tssim(const bool tdxo = 0,
         Alag2x[k] = Alag2;
         eventx[k] = event;
         
-        k++;
+        ++k;
         
         if (event == 1) break;
       }
       
       // generate random censoring due to dropout
-      double C = R::rexp(1.0/rate_C);
-      C = std::round(C);
-      if (C == 0) C = 1;
+      double C = std::round(expo(rng) / rate_C);
+      if (C == 0.0) C = 1.0;
       double time = std::min({T, C, fu});
       
       int J;  // J is the number of treatment cycles
       if (time <= days*followup) {
-        J = static_cast<int>(std::ceil(time/days));
+        J = static_cast<int>(std::ceil(time / days));
         k = k - tpoint + J; // discard treatment cycles after censoring
         tstopx[k-1] = time; // update the ending time and event indicator
-        eventx[k-1] = T == time ? 1 : 0;
-        Zx[k-1] = NA_INTEGER;
-        Ax[k-1] = NA_INTEGER;
+        eventx[k-1] = (T == time) ? 1 : 0;
+        Zx[k-1] = INT_MIN;
+        Ax[k-1] = INT_MIN;
       } else { // add one more record 
         J = followup + 1;
         idx[k] = id;
@@ -494,21 +515,22 @@ List tssim(const bool tdxo = 0,
         tpointx[k] = followup + 1;
         tstartx[k] = days*followup;
         tstopx[k] = time;
-        Lx[k] = (R::runif(0,1) <= R::plogis(gamma0 + gamma1*Alag + 
-          gamma2*Llag + gamma3*bprog + gamma4*trtrand, 0, 1, 1, 0));
+        double probL_final = boost_plogis(gamma0 + gamma1 * Alag + gamma2 * Llag 
+                                            + gamma3 * bprog + gamma4 * trtrand);
+        Lx[k] = (unif(rng) <= probL_final) ? 1 : 0;
         Llagx[k] = L;
-        Zx[k] = NA_INTEGER;
+        Zx[k] = INT_MIN;
         Zlagx[k] = Z;
-        Ax[k] = NA_INTEGER;
+        Ax[k] = INT_MIN;
         Alagx[k] = A;
         Alag2x[k] = Alag;
-        eventx[k] = T == time ? 1 : 0;
+        eventx[k] = (T == time) ? 1 : 0;
         
-        k++;
+        ++k;
       }
       
       // create subject-level survival time and death indicator
-      for (int j=k-J; j<k; j++) {
+      for (int j = k - J; j < k; j++) {
         timeOSx[j] = tstopx[k-1];
         diedx[j] = eventx[k-1];
         censor_timex[j] = fu;
@@ -516,23 +538,19 @@ List tssim(const bool tdxo = 0,
       
       // progression and time to progression (if applicable)
       int pd = 0;
-      double pd_time = NA_REAL;
-      for (int j=k-J; j<k; j++) {
-        if (Zx[j] == 1) {
-          pd = 1; pd_time = tstopx[j]; break;
-        }
+      double pd_time = NaN;
+      for (int j = k - J; j < k; j++) {
+        if (Zx[j] == 1) { pd = 1; pd_time = tstopx[j]; break; }
       }
 
       // switching and time to switching (if applicable)
       int xo = 0;
-      double xo_time = NA_REAL;
-      for (int j=k-J; j<k; j++) {
-        if (Ax[j] == 1) {
-          xo = 1; xo_time = tstopx[j]; break;
-        }
+      double xo_time = NaN;
+      for (int j = k - J; j < k; j++) {
+        if (Ax[j] == 1) { xo = 1; xo_time = tstopx[j]; break; }
       }
       
-      for (int j=k-J; j<k; j++) {
+      for (int j = k - J; j < k; j++) {
         progressedx[j] = pd;
         timePDx[j] = pd_time;
         xox[j] = xo;
@@ -540,60 +558,63 @@ List tssim(const bool tdxo = 0,
       }
       
       // shift disease progression and alternative therapy status downward
-      for (int j=k-J; j<k; j++) {
+      for (int j = k - J; j < k; j++) {
         Zx[j] = Zlagx[j];
         Ax[j] = Alagx[j];
         Alagx[j] = Alag2x[j];
       }
     }
     
-    IntegerVector sub = seq(0, k-1);
-    idx = idx[sub];
-    arrivalTimex = arrivalTimex[sub];
-    trtrandx = trtrandx[sub];
-    bprogx = bprogx[sub];
-    tpointx = tpointx[sub];
-    Lx = Lx[sub];
-    Llagx = Llagx[sub];
-    Zx = Zx[sub];
-    Ax = Ax[sub];
-    Alagx = Alagx[sub];
-    eventx = eventx[sub];
-    diedx = diedx[sub];
-    progressedx = progressedx[sub];
-    xox = xox[sub];
-    tstartx = tstartx[sub];
-    tstopx = tstopx[sub];
-    timeOSx = timeOSx[sub];
-    timePDx = timePDx[sub];
-    xotimex = xotimex[sub];
-    censor_timex = censor_timex[sub];
+    std::vector<int> sub = seqcpp(0, k - 1);
+    subset_in_place(idx, sub);
+    subset_in_place(arrivalTimex, sub);
+    subset_in_place( trtrandx, sub);
+    subset_in_place(bprogx, sub);
+    subset_in_place(tpointx, sub);
+    subset_in_place(tstartx, sub);
+    subset_in_place(tstopx, sub);
+    subset_in_place(Lx, sub);
+    subset_in_place(Llagx, sub);
+    subset_in_place(Zx, sub);
+    subset_in_place(Ax, sub);
+    subset_in_place(Alagx, sub);
+    subset_in_place(eventx, sub);
+    subset_in_place(diedx, sub);
+    subset_in_place(progressedx, sub);
+    subset_in_place(timeOSx, sub);
+    subset_in_place(timePDx, sub);
+    subset_in_place(xox, sub);
+    subset_in_place(xotimex, sub);
+    subset_in_place(censor_timex, sub);
     
-    DataFrame data = DataFrame::create(
-      _["id"] = idx,
-      _["arrival_time"] = arrivalTimex,
-      _["trtrand"] = trtrandx,
-      _["bprog"] = bprogx,
-      _["tpoint"] = tpointx,
-      _["tstart"] = tstartx,
-      _["tstop"] = tstopx,
-      _["L"] = Lx,
-      _["Llag"] = Llagx,
-      _["Z"] = Zx,
-      _["A"] = Ax,
-      _["Alag"] = Alagx,
-      _["event"] = eventx,
-      _["timeOS"] = timeOSx,
-      _["died"] = diedx,
-      _["progressed"] = progressedx,
-      _["timePD"] = timePDx,
-      _["xo"] = xox,
-      _["xotime"] = xotimex,
-      _["censor_time"] = censor_timex
-    );
+    DataFrameCpp data;
+    data.push_back(idx, "id");
+    data.push_back(arrivalTimex, "arrival_time");
+    data.push_back(trtrandx, "trtrand");
+    data.push_back(bprogx, "bprog");
+    data.push_back(tpointx, "tpoint");
+    data.push_back(tstartx, "tstart");
+    data.push_back(tstopx, "tstop");
+    data.push_back(Lx, "L");
+    data.push_back(Llagx, "Llag");
+    data.push_back(Zx, "Z");
+    data.push_back(Ax, "A");
+    data.push_back(Alagx, "Alag");
+    data.push_back(eventx, "event");
+    data.push_back(timeOSx, "timeOS");
+    data.push_back(diedx, "died");
+    data.push_back(progressedx, "progressed");
+    data.push_back(timePDx, "timePD");
+    data.push_back(xox, "xo");
+    data.push_back(xotimex, "xotime");
+    data.push_back(censor_timex, "censor_time");
     
-    result[iter] = data;
+    sims[iter] = std::move(data);
   }
   
-  return result;
+  Rcpp::List out(sims.size());
+  for (std::size_t i = 0; i < sims.size(); ++i) {
+    out[i] = convertDataFrameCppToR(sims[i]);
+  }
+  return out;
 }
