@@ -11,7 +11,7 @@ testthat::test_that("tsegest: logistic g-estimation", {
     pcattrtbprog = 0.25, pcatnotrt = 0.2, pcattrt = 0.1, 
     catmult = 0.5, tdxo = 1, ppoor = 0.1, pgood = 0.04, 
     ppoormet = 0.4, pgoodmet = 0.2, xomult = 1.4188308, 
-    milestone = 546, outputRawDataset = 1, seed = 2000)
+    milestone = 546, seed = 2000)
   
   data1 <- sim1$paneldata %>%
     mutate(visit7on = ifelse(progressed == 1, tstop > timePFSobs + 105, 0))
